<?xml version="1.0" encoding="UTF-8"?>

<!--
 @file docgen.main.xsl
 
 This stylesheet is resonsible for taking a message definition
 file (mxsd file) and producing HTML output.
 This transform is invoked by the HtmlDocWriteHelper class.
-->

<!DOCTYPE xsl:stylesheet [
<!ENTITY eol     "&#10;">
<!ENTITY space   "&#32;">
<!ENTITY nbsp    "&#160;">
]>

<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:xalan="http://xml.apache.org/xslt">

  <xsl:param name="title">Page Title</xsl:param>
  <xsl:param name="filename">file.mxsd</xsl:param>
  <xsl:param name="relativePath"></xsl:param>
  <xsl:param name="outputPath"></xsl:param>
  <xsl:param name="nlsfile">docgen_nls.xml</xsl:param>

  <xsl:output method="html" version="1.0" encoding="UTF-8" media-type="text/html" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:variable name="pathsep" select="'.'" />
  
  <!--
  	Namespace processing.
  	The target namespace is just an attribute of the schema.
  	The namespace prefix is harder to pull out, the algorithm used is:
  	- If there is a target namespace then
  	- loop though all the namespaces until you find the one matching the target
  	- use the prefix for this matching namespace.
  	-->
  <xsl:variable name="namespaceTarget">
  	<xsl:choose>
  		<xsl:when test="/xsd:schema/@targetNamespace">
  			<xsl:value-of select="/xsd:schema/@targetNamespace"/>
  		</xsl:when>
  	</xsl:choose>
  </xsl:variable>
	
  <xsl:variable name="namespacePrefix">
  	<xsl:choose>
  		<xsl:when test="/xsd:schema/@targetNamespace">
			<xsl:for-each select="/xsd:schema/namespace::*">
				<xsl:if test="self::node() = $namespaceTarget">
		  			<xsl:value-of select="local-name()"/>
				</xsl:if>
			</xsl:for-each>
  		</xsl:when>
  	</xsl:choose>
  </xsl:variable>

	<!--
      #          
     #           
    #            
   #             
  #              
 #               
#                
-->
  <xsl:template match="/">

	<html>
	<head>
	
	<!-- Trace to confirm the NLS file -->
	<xsl:comment>
			<xsl:text>NLS File:</xsl:text>
			<xsl:value-of select="$nlsfile" />
    </xsl:comment>
	
	<!-- Trace to confirm the Output Path -->
	<xsl:comment>
			<xsl:text>Output Path:</xsl:text>
			<xsl:value-of select="$outputPath" />
    </xsl:comment>
	
    <link type="text/css" href="{$relativePath}docgen.css" rel="stylesheet"/>
	<title>
	<xsl:value-of select="$title"/>
	</title>
	</head>
	<body>
    <xsl:call-template name="outputPrologue" />
    <xsl:call-template name="messages" />
    <xsl:call-template name="types" />
    <xsl:call-template name="groups" />
    <xsl:call-template name="elementsAndAttributes" />
    </body>
    </html>
    <!--
    -->
  </xsl:template>
  
<!--
  ####   #    #   #####  #####   #    #   #####  #####   #####    ####   #
 #    #  #    #     #    #    #  #    #     #    #    #  #    #  #    #  #
 #    #  #    #     #    #    #  #    #     #    #    #  #    #  #    #  #
 #    #  #    #     #    #####   #    #     #    #####   #####   #    #  #
 #    #  #    #     #    #       #    #     #    #       #   #   #    #  #
  ####    ####      #    #        ####      #    #       #    #   ####   ######
-->
  <xsl:template name="outputPrologue">

    <xsl:comment>
      <xsl:text>outputPrologue</xsl:text>
    </xsl:comment>
    
    <a name="top">
    <h3>
	<xsl:value-of select="$title"/>
	</h3>
	</a>

	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="'MDC'" />
		<xsl:with-param name="target" select="'NoTarget'" />
		<xsl:with-param name="displaytext" select="$title" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />
	
	<xsl:for-each select="/xsd:schema">
		
		<!-- Namespace -->
 		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalnamespaceprefix" />
		<xsl:call-template name="logicalnamespacetarget" />
	  	<xsl:call-template name="bodyinnerend" />

		<!-- Default Element and Attribute Namespace -->
 		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalnamespacedefaultelement" />
		<xsl:call-template name="logicalnamespacedefaultattribute" />
	  	<xsl:call-template name="bodyinnerend" />

		<!-- Defaults -->
 		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicaldefaultblock" />
		<xsl:call-template name="logicaldefaultfinal" />
	  	<xsl:call-template name="bodyinnerend" />

		<!-- Documentation -->
 		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicaldocumentation" />
	  	<xsl:call-template name="bodyinnerend" />

	</xsl:for-each>
  	
	<xsl:call-template name="blockend" />

  </xsl:template>
  
<!--
 #    #  ######   ####    ####     ##     ####   ######   ####
 ##  ##  #       #       #        #  #   #    #  #       #
 # ## #  #####    ####    ####   #    #  #       #####    ####
 #    #  #            #       #  ######  #  ###  #            #
 #    #  #       #    #  #    #  #    #  #    #  #       #    #
 #    #  ######   ####    ####   #    #   ####   ######   ####
-->
  <xsl:template name="messages">

	<xsl:call-template name="blockcomment">
		<xsl:with-param name="text">Messages</xsl:with-param>
	</xsl:call-template>

    <xsl:variable name="fullpath" select="'messages/'"/>

    <xsl:call-template name="writeSectionAnchor">
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="displaytext">
    		<xsl:call-template name="lookupText">
    			<xsl:with-param name="key" select="'messages'" />
    		</xsl:call-template>
    	</xsl:with-param>
    </xsl:call-template>

    <xsl:for-each select="/xsd:schema/xsd:element//*[local-name()='MRMessage']">
		<xsl:apply-templates select=".">
			<xsl:with-param name="prefix" select="concat($fullpath,'message/')" />
		</xsl:apply-templates>
	</xsl:for-each>
  </xsl:template>

<!--
 #####   #   #  #####   ######   ####
   #      # #   #    #  #       #
   #       #    #    #  #####    ####
   #       #    #####   #            #
   #       #    #       #       #    #
   #       #    #       ######   ####
-->
  <xsl:template name="types">

	<xsl:call-template name="blockcomment">
		<xsl:with-param name="text">Types</xsl:with-param>
	</xsl:call-template>

    <xsl:variable name="fullpath" select="'types/'"/>

    <xsl:call-template name="writeSectionAnchor">
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="displaytext">
    		<xsl:call-template name="lookupText">
    			<xsl:with-param name="key" select="'types'" />
    		</xsl:call-template>
    	</xsl:with-param>
    </xsl:call-template>

    <xsl:for-each select="/xsd:schema/xsd:complexType | /xsd:schema/xsd:simpleType">
      <xsl:apply-templates select=".">
		<xsl:with-param name="prefix" select="concat($fullpath,'type/')" />
      </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>

<!--
  ####   #####    ####   #    #  #####    ####
 #    #  #    #  #    #  #    #  #    #  #
 #       #    #  #    #  #    #  #    #   ####
 #  ###  #####   #    #  #    #  #####        #
 #    #  #   #   #    #  #    #  #       #    #
  ####   #    #   ####    ####   #        ####
-->
  <xsl:template name="groups">

	<xsl:call-template name="blockcomment">
		<xsl:with-param name="text">Groups</xsl:with-param>
	</xsl:call-template>

    <xsl:variable name="fullpath" select="'groups/'"/>

    <xsl:call-template name="writeSectionAnchor">
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="displaytext">
    		<xsl:call-template name="lookupText">
    			<xsl:with-param name="key" select="'groups'" />
    		</xsl:call-template>
    	</xsl:with-param>
    </xsl:call-template>
	
    <xsl:for-each select="/xsd:schema/xsd:group | /xsd:schema/xsd:attributeGroup">
      <xsl:apply-templates select=".">
		<xsl:with-param name="prefix" select="concat($fullpath,local-name(),'/')" />
      </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>

<!--
 ######  #        ####     #       ##     #####   ####
 #       #       #         #      #  #      #    #
 #####   #        ####   #####   #    #     #     ####
 #       #            #    #     ######     #         #
 #       #       #    #    #     #    #     #    #    #
 ######  ######   ####           #    #     #     ####
-->
  <xsl:template name="elementsAndAttributes">

	<xsl:call-template name="blockcomment">
		<xsl:with-param name="text">Elements and Attributes</xsl:with-param>
	</xsl:call-template>

    <xsl:variable name="fullpath" select="'elementsAndAttributes/'"/>

    <xsl:call-template name="writeSectionAnchor">
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="displaytext">
    		<xsl:call-template name="lookupText">
    			<xsl:with-param name="key" select="'elementsAndAttributes'" />
    		</xsl:call-template>
    	</xsl:with-param>
    </xsl:call-template>

	<xsl:for-each select="/xsd:schema/xsd:element | /xsd:schema/xsd:attribute">
		<xsl:apply-templates select=".">
			<xsl:with-param name="prefix" select="concat($fullpath,local-name(),'/')" />
		</xsl:apply-templates>
	</xsl:for-each>

  </xsl:template>

  <!--
  	######  #       ######  #    #  ######  #    #   #####
  	#       #       #       ##  ##  #       ##   #     #
  	#####   #       #####   # ## #  #####   # #  #     #
  	#       #       #       #    #  #       #  # #     #
  	#       #       #       #    #  #       #   ##     #
  	######  ######  ######  #    #  ######  #    #     #
  -->
  <xsl:template match="xsd:element[@name]">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
	<xsl:choose>
		<!-- 
			Compound elements have specific rules.
			Modify this line as the rules alter
		-->
		<xsl:when test="xsd:complexType/@mixed='true' and xsd:complexType/xsd:sequence/xsd:element/@maxOccurs='0' and xsd:complexType/xsd:sequence/xsd:element/@minOccurs='0' and starts-with(xsd:complexType/xsd:sequence/xsd:element/@type,'ComIbmMrm_BaseValue') and starts-with(xsd:complexType/xsd:sequence/xsd:element/@name,'ComIbmMrm_21')">
			<xsl:call-template name="processCompoundElement">
		   		<xsl:with-param name="prefix" select="$prefix" />
		   		<xsl:with-param name="displaysuffix" select="$displaysuffix" />
		   		<xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="processElement">
		   		<xsl:with-param name="prefix" select="$prefix" />
		   		<xsl:with-param name="displaysuffix" select="$displaysuffix" />
		   		<xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
	
  </xsl:template>
<!--  
  ####    ####   #    #  #####    ####   #    #  #    #  #####
 #    #  #    #  ##  ##  #    #  #    #  #    #  ##   #  #    #
 #       #    #  # ## #  #    #  #    #  #    #  # #  #  #    #
 #       #    #  #    #  #####   #    #  #    #  #  # #  #    #
 #    #  #    #  #    #  #       #    #  #    #  #   ##  #    #
  ####    ####   #    #  #        ####    ####   #    #  #####
-->
  <xsl:template name="processCompoundElement">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'SBT'"/>

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="@name"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="nameandtype">
  		<xsl:value-of select="@name"/>
		<xsl:text> (</xsl:text>
		<xsl:value-of select="xsd:complexType/xsd:sequence/xsd:element/@type"/>
		<xsl:text>)</xsl:text>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$nameandtype" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />
  	<xsl:call-template name="bodyinnerbegin" />

	<xsl:call-template name="logicalname" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
	<xsl:for-each select="xsd:complexType/xsd:sequence/xsd:element">
		<xsl:call-template name="logicalnillable" />
	</xsl:for-each>
	<xsl:call-template name="logicalabstract" />
		
  	<xsl:call-template name="bodyinnerend" />
  	
	<!-- Value -->
  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:for-each select="xsd:complexType/xsd:sequence/xsd:element">
		<xsl:call-template name="logicaldefaultorfixed" />
	</xsl:for-each>
  	<xsl:call-template name="bodyinnerend" />
  	
	<!-- Occurences -->
	<xsl:if test="$displaysuffix">
	  	<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalminoccurs" />
		<xsl:call-template name="logicalmaxoccurs" />
	  	<xsl:call-template name="bodyinnerend" />
	</xsl:if>

  	<!-- Content -->
  	<xsl:call-template name="bodyinnerbegin" />
  	<xsl:for-each select="xsd:complexType">
		<xsl:call-template name="logicalcomplexcontent" />
		<xsl:call-template name="logicalcomplextypecontent" />
	  	<xsl:for-each select="xsd:sequence">
			<xsl:call-template name="logicalgroupreference" />
	  	</xsl:for-each>
		<xsl:call-template name="logicalmixed" />
  	</xsl:for-each>
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
	<xsl:for-each select="xsd:complexType">
		<xsl:call-template name="logicaldocumentationnotitle" />
	</xsl:for-each>
  	<xsl:call-template name="bodyinnerend" />
  	
	<xsl:call-template name="blockend" />
  </xsl:template>
  
  <xsl:template name="processElement">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'ED'"/>

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="@name"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="nameandtype">
  		<xsl:value-of select="@name"/>
		<xsl:text> (</xsl:text>
		<xsl:call-template name="getelementtype"/>
		<xsl:text>)</xsl:text>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$nameandtype" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />
  	<xsl:call-template name="bodyinnerbegin" />

	<xsl:call-template name="logicalname" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
		
	<xsl:choose>
		<xsl:when test="xsd:simpleType">
			<xsl:call-template name="logicallocalsimpletype" />
		</xsl:when>
		<xsl:when test="xsd:complexType/xsd:simpleContent">
			<xsl:call-template name="logicallocalcomplextypesimplecontent" />
		</xsl:when>
		<xsl:when test="xsd:complexType/xsd:complexContent">
			<xsl:call-template name="logicallocalcomplextypecomplexcontent" />
		</xsl:when>
		<xsl:when test="xsd:complexType">
			<xsl:call-template name="logicallocalcomplextype" />
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="logicaltype" />
		</xsl:otherwise>
	</xsl:choose>
	
	<!-- Namespace -->
	<xsl:call-template name="logicalnamespaceelement"/>

  	<xsl:call-template name="bodyinnerend" />
  	
	<!-- Occurences -->
	<xsl:if test="$displaysuffix">
	  	<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalminoccurs" />
		<xsl:call-template name="logicalmaxoccurs" />
	  	<xsl:call-template name="bodyinnerend" />
	</xsl:if>

	<!-- Value -->
  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldefaultorfixed" />
	<xsl:call-template name="logicalnillable" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<!-- Substitution settings -->
  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalfinal" />
	<xsl:call-template name="logicalblock" />
	<xsl:call-template name="logicalsubstitutiongroup" />
	<xsl:call-template name="logicalabstract" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
	<xsl:call-template name="blockend" />

   	<xsl:apply-templates>
   		<xsl:with-param name="prefix" select="$prefix" />
   		<xsl:with-param name="displaysuffix" select="$displaypath" />
   		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
   	</xsl:apply-templates>
      	
  </xsl:template>

		<!-- 
			This is quite hard. 
			Have to follow the reference, possibly to
			another file, and then extract the type.
			This could possibly done with some changes to the 
			format of globals.lut.xml
		-->
		<xsl:template name="getrefelementtype">
			<xsl:value-of select="'TBD'"/>
		</xsl:template>

	<xsl:template name="getelementtype">
	<xsl:choose>
		<xsl:when test="xsd:simpleType/xsd:restriction/@base">
  			<xsl:value-of select="concat(xsd:simpleType/xsd:restriction/@base,' -')" />
		</xsl:when>
		<xsl:when test="xsd:simpleType/xsd:list/@itemType">
  			<xsl:value-of select="concat(xsd:simpleType/xsd:list/@itemType,' -')" />
		</xsl:when>
		<xsl:when test="xsd:simpleType/xsd:union">
  			<xsl:value-of select="concat('union',' -')" />
		</xsl:when>
		<xsl:when test="xsd:complexType/xsd:simpleContent">
  			<xsl:value-of select="concat(xsd:complexType/xsd:simpleContent/xsd:extension/@base,' +')" />
		</xsl:when>
		<xsl:when test="xsd:complexType/xsd:complexContent/xsd:extension/@base">
  			<xsl:value-of select="concat(xsd:complexType/xsd:complexContent/xsd:extension/@base,' +')" />
		</xsl:when>
		<xsl:when test="xsd:complexType/xsd:complexContent/xsd:restriction/@base">
  			<xsl:value-of select="concat(xsd:complexType/xsd:complexContent/xsd:restriction/@base,' -')" />
		</xsl:when>
		<xsl:when test="xsd:complexType">
  			<xsl:value-of select="$localComplexTypeString" />
		</xsl:when>
		<xsl:otherwise>
  			<xsl:call-template name="lookupglobaltype">
  				<xsl:with-param name="type" select="@type" />
  			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
	</xsl:template>

	<xsl:template name="getattributetype">
		<xsl:choose>
			<xsl:when test="xsd:simpleType/xsd:restriction/@base">
  				<xsl:value-of select="concat(xsd:simpleType/xsd:restriction/@base,' -')" />
			</xsl:when>
			<xsl:when test="xsd:simpleType/xsd:list/@itemType">
  				<xsl:value-of select="concat(xsd:simpleType/xsd:list/@itemType,' -')" />
			</xsl:when>
			<xsl:when test="xsd:simpleType/xsd:union">
  				<xsl:value-of select="concat('union',' -')" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="lookupglobaltype">
					<xsl:with-param name="type" select="@type" />
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

  <!--
  	######  #       ######  #    #  ######  #    #   #####
  	#       #       #       ##  ##  #       ##   #     #
  	#####   #       #####   # ## #  #####   # #  #     #
  	#       #       #       #    #  #       #  # #     #
  	#       #       #       #    #  #       #   ##     #
  	######  ######  ######  #    #  ######  #    #     #
  -->
  <xsl:template match="xsd:element[@ref]">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniqueref" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'EU'" />

    <xsl:variable name="type" select="@ref"/>

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$type"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="nameandtype">
  		<xsl:value-of select="@ref"/>
  		<!--
		<xsl:text> (</xsl:text>
		<xsl:call-template name="getrefelementtype"/>
		<xsl:text>)</xsl:text>
		-->
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$nameandtype" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalreferencenameelement" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
  	<xsl:call-template name="bodyinnerend" />

	<!-- Occurences -->
  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalminoccurs" />
	<xsl:call-template name="logicalmaxoccurs" />
  	<xsl:call-template name="bodyinnerend" />
	
	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

   	<xsl:apply-templates>
   		<xsl:with-param name="prefix" select="$prefix" />
   		<xsl:with-param name="displaysuffix" select="$displaypath" />
   		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
   	</xsl:apply-templates>
      	
  </xsl:template>
  
<!--
 #    #  #####   #    #  ######   ####    ####     ##     ####   ######  
 ##  ##  #    #  ##  ##  #       #       #        #  #   #    #  #       
 # ## #  #    #  # ## #  #####    ####    ####   #    #  #       #####   
 #    #  #####   #    #  #            #       #  ######  #  ###  #       
 #    #  #   #   #    #  #       #    #  #    #  #    #  #    #  #       
 #    #  #    #  #    #  ######   ####    ####   #    #   ####   ######  
-->
  <xsl:template match="*[local-name()='MRMessage']">
  	<xsl:param name="prefix" />

  	<xsl:variable name="atname" select="ancestor::xsd:element/@name" />
  	<!--
  	<xsl:variable name="attype" select="ancestor::xsd:element/@type"/>
  	-->
  	<xsl:variable name="attype">
  		<xsl:choose>
  			<xsl:when test="ancestor::xsd:element/xsd:complexType">
	  			<xsl:value-of select="$localComplexTypeString" />
  			</xsl:when>
  			<!--
  			-->
  			<xsl:when test="ancestor::xsd:element/@type">
  				<xsl:for-each select="ancestor::xsd:element">
					<xsl:call-template name="getelementtype"/>
  				</xsl:for-each>
  				<!--
  				<xsl:value-of select="ancestor::xsd:element/@type"/>
  				-->
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="'No Type'"/>
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>
  	<!--
  	-->
  	<xsl:variable name="icontype" select="'MO'" />
  	<xsl:variable name="fullpath" select="concat($prefix,$atname,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="concat($atname,' (',$attype,')')" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />
  	
	<xsl:for-each select="ancestor::xsd:element">
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalname" />
        <xsl:call-template name="logicalmessagealias" />
		<xsl:call-template name="bodyinnerend" />
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicaldocumentation" />
		<xsl:call-template name="bodyinnerend" />
	</xsl:for-each>

  	<xsl:call-template name="blockend" />

  </xsl:template>

<!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######
  #  #      #       #    #    #     #    #    #  #    #     #    #
 #    #     #       #    #    #     #    #####   #    #     #    #####
 ######     #       #    #####      #    #    #  #    #     #    #
 #    #     #       #    #   #      #    #    #  #    #     #    #
 #    #     #       #    #    #     #    #####    ####      #    ######
-->
  <xsl:template match="xsd:attribute[@name]">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

    <xsl:variable name="icontype" select="'AD'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="@name"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="nameandtype">
  		<xsl:value-of select="@name"/>
		<xsl:text> (</xsl:text>
		<xsl:call-template name="getattributetype"/>
		<xsl:text>)</xsl:text>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$nameandtype" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />
  	<xsl:call-template name="bodyinnerbegin" />

	<xsl:call-template name="logicalname" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
	<xsl:choose>
		<xsl:when test="xsd:simpleType">
			<xsl:call-template name="logicallocalsimpletype" />
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="logicaltype" />
		</xsl:otherwise>
	</xsl:choose>

	<!-- Namespace -->
	<xsl:call-template name="logicalnamespaceattribute"/>

  	<xsl:call-template name="bodyinnerend" />

	<!-- Value -->
  	<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicaldefaultorfixed" />
  	<xsl:call-template name="bodyinnerend" />
	
	<!-- Usage -->
	<xsl:if test="$displaysuffix">
	  	<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalusage" />
	  	<xsl:call-template name="bodyinnerend" />
	</xsl:if>
	
	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

   	<xsl:apply-templates>
   		<xsl:with-param name="prefix" select="$prefix" />
   		<xsl:with-param name="displaysuffix" select="$displaypath" />
   		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
   	</xsl:apply-templates>
      	
  </xsl:template>

<!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######
  #  #      #       #    #    #     #    #    #  #    #     #    #
 #    #     #       #    #    #     #    #####   #    #     #    #####
 ######     #       #    #####      #    #    #  #    #     #    #
 #    #     #       #    #   #      #    #    #  #    #     #    #
 #    #     #       #    #    #     #    #####    ####      #    ######
-->
  <xsl:template match="xsd:attribute[@ref]">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniqueref" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'AU'" />

    <xsl:variable name="type" select="@ref"/>

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$type"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$type" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalreferencenameattribute" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
	<xsl:call-template name="logicalusage" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

   	<xsl:apply-templates>
   		<xsl:with-param name="prefix" select="$prefix" />
   		<xsl:with-param name="displaysuffix" select="$displaypath" />
   		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
   	</xsl:apply-templates>
      	
  </xsl:template>

<!--
  ####   #####    ####   #    #  #####   
 #    #  #    #  #    #  #    #  #    #  
 #       #    #  #    #  #    #  #    #  
 #  ###  #####   #    #  #    #  #####   
 #    #  #   #   #    #  #    #  #       
  ####   #    #   ####    ####   #       
-->
  <xsl:template match="xsd:group[@name]">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'MGD'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="@name"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="@name" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalname" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
	<xsl:call-template name="logicalglobalcontentmodel" />
	<xsl:call-template name="logicalglobalgroupcontent" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:call-template name="processGroup">
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:call-template>

  </xsl:template>
  
  <!--
  ####   #####    ####   #    #  #####   
 #    #  #    #  #    #  #    #  #    #  
 #       #    #  #    #  #    #  #    #  
 #  ###  #####   #    #  #    #  #####   
 #    #  #   #   #    #  #    #  #       
  ####   #    #   ####    ####   #       
-->
  <xsl:template match="xsd:group[@ref]">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniqueref" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'MGU'" />

    <xsl:variable name="type" select="@ref"/>

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$type"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$type" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalreferencenamegroup" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
  	<xsl:call-template name="bodyinnerend" />

	<!-- Occurences -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalminoccurs" />
	<xsl:call-template name="logicalmaxoccurs" />
  	<xsl:call-template name="bodyinnerend" />
	
	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

   	<xsl:apply-templates>
   		<xsl:with-param name="prefix" select="$prefix" />
   		<xsl:with-param name="displaysuffix" select="$displaypath" />
   		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
   	</xsl:apply-templates>
      	
  </xsl:template>
  	
<!--
  ####   #####    ####   #    #  #####   
 #    #  #    #  #    #  #    #  #    #  
 #       #    #  #    #  #    #  #    #  
 #  ###  #####   #    #  #    #  #####   
 #    #  #   #   #    #  #    #  #       
  ####   #    #   ####    ####   #       
-->
  <xsl:template match="xsd:group[@ref]" mode="notreeentry">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
   		<xsl:with-param name="displaysuffix" select="$displaysuffix" />
   		<xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
  	</xsl:apply-templates>

  </xsl:template>

<!--
 #####   #####    ####    ####   ######   ####    ####    ####   #####    ####  
 #    #  #    #  #    #  #    #  #       #       #       #    #  #    #  #    # 
 #    #  #    #  #    #  #       #####    ####    ####   #       #    #  #    # 
 #####   #####   #    #  #       #            #       #  #  ###  #####   #    # 
 #       #   #   #    #  #    #  #       #    #  #    #  #    #  #   #   #    # 
 #       #    #   ####    ####   ######   ####    ####    ####   #    #   ####  
-->  
  <xsl:template name="processGroup">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:for-each select="*">
  		<xsl:for-each select="*">
  			<xsl:apply-templates select=".">
  				<xsl:with-param name="prefix" select="$prefix" />
  				<xsl:with-param name="displaysuffix" select="$displaysuffix" />
  				<xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
  			</xsl:apply-templates>
  		</xsl:for-each>
  	</xsl:for-each>

  </xsl:template>

<!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######   ####  
  #  #      #       #    #    #     #    #    #  #    #     #    #       #    # 
 #    #     #       #    #    #     #    #####   #    #     #    #####   #      
 ######     #       #    #####      #    #    #  #    #     #    #       #  ### 
 #    #     #       #    #   #      #    #    #  #    #     #    #       #    # 
 #    #     #       #    #    #     #    #####    ####      #    ######   ####  
-->
  <xsl:template match="xsd:attributeGroup[@name]">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'AGD'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="@name"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="@name" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalname" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:call-template name="processAttributeGroup">
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:call-template>
  </xsl:template>

<!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######   ####  
  #  #      #       #    #    #     #    #    #  #    #     #    #       #    # 
 #    #     #       #    #    #     #    #####   #    #     #    #####   #      
 ######     #       #    #####      #    #    #  #    #     #    #       #  ### 
 #    #     #       #    #   #      #    #    #  #    #     #    #       #    # 
 #    #     #       #    #    #     #    #####    ####      #    ######   ####  
-->
  <xsl:template match="xsd:attributeGroup[@ref]">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniqueref" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'AGU'" />

    <xsl:variable name="type" select="@ref"/>

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$type"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$type" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />
  	
  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalreferencenameattributegroup" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
  	<xsl:call-template name="bodyinnerend" />
	
	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:apply-templates>

  </xsl:template>

<!--
 #####   #####    ####    ####   ######   ####    ####     ##     #####   ##### 
 #    #  #    #  #    #  #    #  #       #       #        #  #      #       #   
 #    #  #    #  #    #  #       #####    ####    ####   #    #     #       #   
 #####   #####   #    #  #       #            #       #  ######     #       #   
 #       #   #   #    #  #    #  #       #    #  #    #  #    #     #       #   
 #       #    #   ####    ####   ######   ####    ####   #    #     #       #   
-->  
  <xsl:template name="processAttributeGroup">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaysuffix" />
  		<xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
  	</xsl:apply-templates>
  </xsl:template>

  <!--  
  ####      #    #    #  #####   #       ######   #####   #   #  #####   ###### 
 #          #    ##  ##  #    #  #       #          #      # #   #    #  #      
  ####      #    # ## #  #    #  #       #####      #       #    #    #  #####  
      #     #    #    #  #####   #       #          #       #    #####   #      
 #    #     #    #    #  #       #       #          #       #    #       #      
  ####      #    #    #  #       ######  ######     #       #    #       ###### 
-->
  <xsl:template match="xsd:simpleType" mode="unionmembertype">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

    <xsl:call-template name="blockcomment">
        <xsl:with-param name="text">
            <xsl:text>prefix </xsl:text>
            <xsl:value-of select="$prefix"/>
            <xsl:text>displaysuffix </xsl:text>
            <xsl:value-of select="$displaysuffix"/>
            <xsl:text>anchorsuffix </xsl:text>
            <xsl:value-of select="$anchorsuffix"/>
        </xsl:with-param>
    </xsl:call-template>

    <xsl:apply-templates select="xsd:list | xsd:union">
        <xsl:with-param name="prefix" select="$prefix" />
        <xsl:with-param name="displaysuffix" select="$displaysuffix" />
        <xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
    </xsl:apply-templates>

  </xsl:template>

  <!--  
  ####      #    #    #  #####   #       ######   #####   #   #  #####   ###### 
 #          #    ##  ##  #    #  #       #          #      # #   #    #  #      
  ####      #    # ## #  #    #  #       #####      #       #    #    #  #####  
      #     #    #    #  #####   #       #          #       #    #####   #      
 #    #     #    #    #  #       #       #          #       #    #       #      
  ####      #    #    #  #       ######  ######     #       #    #       ###### 
-->
  <xsl:template match="xsd:simpleType">
    <xsl:param name="prefix" />
    <xsl:param name="displaysuffix" />
    <xsl:param name="anchorsuffix" />

  	<xsl:variable name="myname">
  		<xsl:choose>
  			<xsl:when test="@name">
  				<xsl:value-of select="@name" />
  			</xsl:when>
            <xsl:when test="xsd:restriction">
                <xsl:value-of select="concat(xsd:restriction/@base,' -')" />
            </xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="$localSimpleTypeString" />
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquename" />
  	</xsl:variable>

  	<xsl:variable name="icontype">
  		<xsl:choose>
  			<xsl:when test="xsd:restriction">
  				<xsl:value-of select="'STDA'" />
  			</xsl:when>
  			<xsl:when test="xsd:list">
  				<xsl:value-of select="'STDL'" />
  			</xsl:when>
  			<xsl:when test="xsd:union">
  				<xsl:value-of select="'STDU'" />
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="'NONE'" />
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>
 
  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$myname"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$myname" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

  	<xsl:call-template name="bodyinnerbegin" />
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$nameString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="$myname" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$unnamedString" />
  	</xsl:call-template>
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>

	<xsl:choose>
		<xsl:when test="xsd:restriction">
			<xsl:call-template name="logicalsimpletypebasetype" />
  		</xsl:when>
  		<xsl:when test="xsd:list">
			<xsl:call-template name="logicalsimpletypeitemtype" />
  		</xsl:when>
  		<xsl:when test="xsd:union">
  			<!-- Do nothing -->
  		</xsl:when>
  		<xsl:otherwise>
  			<!-- Do nothing -->
  		</xsl:otherwise>
  	</xsl:choose>

	<xsl:call-template name="logicalsimpletypevariety" />
  	<xsl:call-template name="bodyinnerend" />

	<xsl:call-template name="processSimpleTypeFacets" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

    <xsl:apply-templates select="xsd:list | xsd:union">
        <xsl:with-param name="prefix" select="$prefix" />
        <xsl:with-param name="displaysuffix" select="$displaypath" />
        <xsl:with-param name="anchorsuffix" select="$anchorpath" />
    </xsl:apply-templates>

    </xsl:template>

  <!--
  ####    ####   #    #  #####   #       ######  #    #   #####   #   #  #####  
 #    #  #    #  ##  ##  #    #  #       #        #  #      #      # #   #    # 
 #       #    #  # ## #  #    #  #       #####     ##       #       #    #    # 
 #       #    #  #    #  #####   #       #         ##       #       #    #####  
 #    #  #    #  #    #  #       #       #        #  #      #       #    #      
  ####    ####   #    #  #       ######  ######  #    #     #       #    #      
-->  
  <xsl:template match="xsd:complexType">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />
  	
  	<xsl:variable name="myname">
  		<xsl:choose>
  			<xsl:when test="@name">
  				<xsl:value-of select="@name" />
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="$localComplexTypeString" />
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>

  	<xsl:variable name="uniqueid">
		<xsl:call-template name="uniquename" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'CTD'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$myname"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$myname" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />
  	<xsl:call-template name="bodyinnerbegin" />

  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$nameString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="$myname" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$unnamedString" />
  	</xsl:call-template>

	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
  	

	<xsl:call-template name="logicalbasetype" />
	<xsl:call-template name="logicalderivedby" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Content -->  	
  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalcomplexcontent" />
	<xsl:call-template name="logicalcomplextypecontent" />
	<xsl:call-template name="logicalmixed" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<!-- Occurences -->
	<xsl:choose>
		<xsl:when test="xsd:sequence">
		  	<xsl:for-each select="xsd:sequence">
			  	<xsl:call-template name="bodyinnerbegin" />
				<xsl:call-template name="logicalminoccurs" />
				<xsl:call-template name="logicalmaxoccurs" />
			  	<xsl:call-template name="bodyinnerend" />
		  	</xsl:for-each>
		</xsl:when>
		<xsl:when test="xsd:group/@ref">
		  	<xsl:for-each select="xsd:group">
			  	<xsl:call-template name="bodyinnerbegin" />
				<xsl:call-template name="logicalminoccurs" />
				<xsl:call-template name="logicalmaxoccurs" />
			  	<xsl:call-template name="bodyinnerend" />
		  	</xsl:for-each>
		</xsl:when>
		<xsl:when test="xsd:complexContent/*/xsd:sequence or xsd:complexContent/*/xsd:choice">
		  	<xsl:for-each select="xsd:complexContent/*/xsd:sequence | xsd:complexContent/*/xsd:choice">
			  	<xsl:call-template name="bodyinnerbegin" />
				<xsl:call-template name="logicalminoccurs" />
				<xsl:call-template name="logicalmaxoccurs" />
			  	<xsl:call-template name="bodyinnerend" />
		  	</xsl:for-each>
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="blockcomment">
				<xsl:with-param name="text">
					<xsl:text>DO NOTHING:</xsl:text>
					<xsl:value-of select="local-name()" />
				</xsl:with-param>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
  	
	<!-- Substitution Settings -->  	
  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalfinal" />
	<xsl:call-template name="logicalblock" />
	<xsl:call-template name="logicalabstract" />
  	<xsl:call-template name="bodyinnerend" />
  	
	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:for-each select="*">
  		<xsl:choose>
  			<xsl:when test="local-name()='sequence' or local-name()='choice' or local-name()='all' or local-name()='group'">
  				<xsl:apply-templates select="." mode="notreeentry">
  					<xsl:with-param name="prefix" select="$prefix" />
  					<xsl:with-param name="displaysuffix" select="$displaypath" />
  					<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  				</xsl:apply-templates>
  			</xsl:when>
  			<xsl:when test="local-name()='element' or local-name()='attribute' or local-name()='attributeGroup' or local-name()='anyAttribute'">
  				<xsl:apply-templates select=".">
  					<xsl:with-param name="prefix" select="$prefix" />
  					<xsl:with-param name="displaysuffix" select="$displaypath" />
  					<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  				</xsl:apply-templates>
  			</xsl:when>
  			<xsl:when test="local-name()='simpleContent'">
  				<xsl:apply-templates select="xsd:extension">
  					<xsl:with-param name="prefix" select="$prefix" />
  					<xsl:with-param name="displaysuffix" select="$displaypath" />
  					<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  				</xsl:apply-templates>
  			</xsl:when>
  			<xsl:when test="local-name()='complexContent'">
  				<xsl:apply-templates select="xsd:extension/*/*|xsd:extension/xsd:attribute|xsd:extension/xsd:anyAttribute">
  					<xsl:with-param name="prefix" select="$prefix" />
  					<xsl:with-param name="displaysuffix" select="$displaypath" />
  					<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  				</xsl:apply-templates>
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:call-template name="blockcomment">
  					<xsl:with-param name="text">
  						<xsl:text>DO NOTHING:</xsl:text>
  						<xsl:value-of select="local-name()" />
  					</xsl:with-param>
  				</xsl:call-template>
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:for-each>

  </xsl:template>

<!--
 #####   #####    ####    ####   ######   ####    ####   #    #  ######  #    #
 #    #  #    #  #    #  #    #  #       #       #       ##  ##  #       ##  ##
 #    #  #    #  #    #  #       #####    ####    ####   # ## #  #####   # ## #
 #####   #####   #    #  #       #            #       #  #    #  #       #    #
 #       #   #   #    #  #    #  #       #    #  #    #  #    #  #       #    #
 #       #    #   ####    ####   ######   ####    ####   #    #  ######  #    #
-->
   <xsl:template name="processMemberTypes">
       <xsl:param name="prefix" />
       <xsl:param name="displaysuffix" />
       <xsl:param name="anchorsuffix" />
       <xsl:param name="types" /> <!-- A space separated list of types -->

          <xsl:variable name="sb" select="substring-before($types,' ')" />

          <xsl:variable name="type">
              <xsl:choose>
                  <xsl:when test="$sb">
                      <xsl:value-of select="$sb" />
                  </xsl:when>
                  <xsl:otherwise>
                      <xsl:value-of select="$types" />
                  </xsl:otherwise>
              </xsl:choose>
          </xsl:variable>

          <xsl:variable name="uniqueid">
              <xsl:call-template name="uniquemembertypeid">
                  <xsl:with-param name="membertype" select="$type"/>
              </xsl:call-template>
          </xsl:variable>

          <xsl:variable name="icontype" select="'STDU'" />

          <xsl:variable name="displaypath">
              <xsl:call-template name="appendtosuffix">
                  <xsl:with-param name="suffix" select="$displaysuffix" />
                  <xsl:with-param name="id" select="$type"/>
              </xsl:call-template>
          </xsl:variable>

          <xsl:variable name="anchorpath">
              <xsl:call-template name="appendtosuffix">
                  <xsl:with-param name="suffix" select="$anchorsuffix" />
                  <xsl:with-param name="id" select="$uniqueid"/>
              </xsl:call-template>
          </xsl:variable>

          <xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

          <xsl:call-template name="blockstart" />

          <xsl:call-template name="writeAnchor">
              <xsl:with-param name="icon" select="$icontype" />
              <xsl:with-param name="target" select="$fullpath" />
              <xsl:with-param name="displaytext" select="$type" />
          </xsl:call-template>

          <xsl:call-template name="blockmiddle" />
          <xsl:call-template name="bodyinnerbegin" />
          <!-- Details here - TBD 
          - name
          - fullname
          - member type with link to type definition if possible
          -->
          <xsl:call-template name="displayValue">
              <xsl:with-param name="key" select="$nameString" />
              <xsl:with-param name="value">
                  <xsl:value-of select="$type" />
              </xsl:with-param>
              <xsl:with-param name="default" select="$unnamedString" />
          </xsl:call-template>

          <xsl:call-template name="logicalfullname">
              <xsl:with-param name="fullname" select="$displaypath"/>
          </xsl:call-template>

          <xsl:call-template name="logicalmembertype">
              <xsl:with-param name="membertype" select="$type"/>
          </xsl:call-template>

          <xsl:call-template name="bodyinnerend" />

          <xsl:call-template name="blockend" />



          <xsl:call-template name="blockcomment">
              <xsl:with-param name="text">
                  <xsl:text>processMemberTypes: </xsl:text>
                  <xsl:value-of select="$type"/>
              </xsl:with-param>
          </xsl:call-template>

          <xsl:apply-templates select="/xsd:schema/xsd:simpleType[@name=$type]" mode="unionmembertype">
              <xsl:with-param name="prefix" select="$prefix" />
              <xsl:with-param name="displaysuffix" select="$displaysuffix" />
              <xsl:with-param name="anchorsuffix" select="concat($anchorsuffix,$pathsep,$uniqueid)" />
          </xsl:apply-templates>

          <xsl:if test="$sb">
              <xsl:call-template name="processMemberTypes">
                  <xsl:with-param name="prefix" select="$prefix" />
                  <xsl:with-param name="displaysuffix" select="$displaysuffix" />
                  <xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
                  <xsl:with-param name="types" select="substring-after($types,' ')" />
              </xsl:call-template>
          </xsl:if>

       </xsl:template>

<!--
 #    #  ######  #    #  #####   ######  #####    #####   #   #  #####   ######
 ##  ##  #       ##  ##  #    #  #       #    #     #      # #   #    #  #
 # ## #  #####   # ## #  #####   #####   #    #     #       #    #    #  #####
 #    #  #       #    #  #    #  #       #####      #       #    #####   #
 #    #  #       #    #  #    #  #       #   #      #       #    #       #
 #    #  ######  #    #  #####   ######  #    #     #       #    #       ######
-->
  <xsl:template match="@memberTypes">
      <xsl:param name="prefix" />
      <xsl:param name="displaysuffix" />
      <xsl:param name="anchorsuffix" />
  	
      <xsl:call-template name="blockcomment">
          <xsl:with-param name="text">@memberTypes</xsl:with-param>
      </xsl:call-template>

      <xsl:call-template name="processMemberTypes">
          <xsl:with-param name="prefix" select="$prefix" />
          <xsl:with-param name="displaysuffix" select="$displaysuffix" />
          <xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
          <xsl:with-param name="types" select="." />
      </xsl:call-template>
	
  </xsl:template>

<!--
 #    #  #    #     #     ####   #    #
 #    #  ##   #     #    #    #  ##   #
 #    #  # #  #     #    #    #  # #  #
 #    #  #  # #     #    #    #  #  # #
 #    #  #   ##     #    #    #  #   ##
  ####   #    #     #     ####   #    #
-->
  <xsl:template match="xsd:union">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

    <xsl:call-template name="blockcomment">
        <xsl:with-param name="text">Union</xsl:with-param>
    </xsl:call-template>

    <!--
    Can have either
    - one memberTypes attribute
    - multiple local simpleType definitions
    -->

       <xsl:apply-templates select="@memberTypes">
           <xsl:with-param name="prefix" select="$prefix" />
           <xsl:with-param name="displaysuffix" select="$displaysuffix" />
           <xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
       </xsl:apply-templates>

       <xsl:apply-templates select="xsd:simpleType">
           <xsl:with-param name="prefix" select="$prefix" />
           <xsl:with-param name="displaysuffix" select="$displaysuffix" />
           <xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
       </xsl:apply-templates>

    </xsl:template>

<!--
 #          #     ####    #####
 #          #    #          #
 #          #     ####      #
 #          #         #     #
 #          #    #    #     #
 ######     #     ####      #
-->
  <xsl:template match="xsd:list">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

    <xsl:call-template name="blockcomment">
        <xsl:with-param name="text">List</xsl:with-param>
    </xsl:call-template>

  </xsl:template>

<!--
   ##    #    #   #   #      
  #  #   ##   #    # #       
 #    #  # #  #     #        
 ######  #  # #     #        
 #    #  #   ##     #        
 #    #  #    #     #        
-->
  <xsl:template match="xsd:any">
    <xsl:param name="prefix" />
    <xsl:param name="displaysuffix" />
    <xsl:param name="anchorsuffix" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquewildcard" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'WE'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$wildcardElementString" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
  		<xsl:with-param name="icon" select="$icontype" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="displaytext" select="$wildcardElementString" />
  	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
	<xsl:call-template name="logicalnamespace" />
	<xsl:call-template name="logicalprocesscontents" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Ocurences -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalminoccurs" />
	<xsl:call-template name="logicalmaxoccurs" />
  	<xsl:call-template name="bodyinnerend" />
  	
	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
   ##    #    #   #   #    ##     #####   #####  #####      #    #####   #    # 
  #  #   ##   #    # #    #  #      #       #    #    #     #    #    #  #    # 
 #    #  # #  #     #    #    #     #       #    #    #     #    #####   #    # 
 ######  #  # #     #    ######     #       #    #####      #    #    #  #    # 
 #    #  #   ##     #    #    #     #       #    #   #      #    #    #  #    # 
 #    #  #    #     #    #    #     #       #    #    #     #    #####    ####  
-->
  <xsl:template match="xsd:anyAttribute">
    <xsl:param name="prefix"/>
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquewildcard" />
  	</xsl:variable>

	<xsl:variable name="icontype" select="'WA'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$wildcardAttributeString"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

	<xsl:call-template name="writeAnchor">
		<xsl:with-param name="icon" select="$icontype" />
		<xsl:with-param name="target" select="$fullpath" />
		<xsl:with-param name="displaytext" select="$wildcardAttributeString" />
	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalnamespace" />
	<xsl:call-template name="logicalprocesscontents" />
  	<xsl:call-template name="bodyinnerend" />

  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />

  	<xsl:call-template name="blockend" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
  ####   ######   ####     #      ####   #    #   ####     #       ##    #      
 #       #       #    #    #     #    #  #    #  #    #    #      #  #   #      
  ####   #####   #    #  #####   #       ######  #    #  #####   #    #  #      
      #  #       #  # #    #     #       #    #  #    #    #     ######  #      
 #    #  #       #   #     #     #    #  #    #  #    #    #     #    #  #      
  ####   ######   ### #           ####   #    #   ####           #    #  ###### 
-->
  <xsl:template match="xsd:sequence|xsd:choice|xsd:all" mode="notreeentry">
      <xsl:param name="prefix" />
      <xsl:param name="displaysuffix" />
      <xsl:param name="anchorsuffix" />

                 <!--
                     Embedded simple types have specific rules.
                     Modify this line as the rules alter
                 -->
         <xsl:for-each select="*">
             <xsl:choose>
                 <xsl:when test="local-name(..)='sequence' and local-name()='element' and @maxOccurs='0' and @minOccurs='0' and starts-with(@type,'ComIbmMrm_Anon')">
                     <xsl:apply-templates select="." mode="embeddedsimpletype">
                         <xsl:with-param name="prefix" select="$prefix" />
                         <xsl:with-param name="displaysuffix" select="$displaysuffix" />
                         <xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
                     </xsl:apply-templates>
                 </xsl:when>
                 <xsl:otherwise>
                     <xsl:apply-templates select=".">
                         <xsl:with-param name="prefix" select="$prefix" />
                         <xsl:with-param name="displaysuffix" select="$displaysuffix" />
                         <xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
                     </xsl:apply-templates>
                 </xsl:otherwise>
             </xsl:choose>
         </xsl:for-each>
<!--
  -->
  </xsl:template>

  <!--
  ####   ######   ####   #    #  ######  #    #   ####   ######     
 #       #       #    #  #    #  #       ##   #  #    #  #          
  ####   #####   #    #  #    #  #####   # #  #  #       #####      
      #  #       #  # #  #    #  #       #  # #  #       #          
 #    #  #       #   #   #    #  #       #   ##  #    #  #          
  ####   ######   ### #   ####   ######  #    #   ####   ######     
-->
  <xsl:template match="xsd:sequence">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquewildcard" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'MGS'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$sequenceString"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
  		<xsl:with-param name="icon" select="$icontype" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="displaytext" select="$sequenceString" />
  	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicallocalcontentmodel" />
	<xsl:call-template name="logicallocalgroupcontent" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Occurences -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalminoccurs" />
	<xsl:call-template name="logicalmaxoccurs" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:apply-templates>
  </xsl:template>

  <!--
  ######  #       ######  #    #  ######  #    #   #####
  #       #       #       ##  ##  #       ##   #     #
  #####   #       #####   # ## #  #####   # #  #     #
  #       #       #       #    #  #       #  # #     #
  #       #       #       #    #  #       #   ##     #
  ######  ######  ######  #    #  ######  #    #     #
  -->
  <xsl:template match="xsd:element" mode="embeddedsimpletype">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

	<!--
	<xsl:call-template name="blockcomment">
		<xsl:with-param name="text">Embedded Simple Type</xsl:with-param>
	</xsl:call-template>
	-->
	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquewildcard" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'EST'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="@type"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid"/>
  		</xsl:call-template>
  	</xsl:variable>
  	
  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
  		<xsl:with-param name="icon" select="$icontype" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="displaytext" select="@type" />
  	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

  	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalembeddedname"/>
	<xsl:call-template name="logicalfullname">
		<xsl:with-param name="fullname" select="$displaypath"/>
	</xsl:call-template>
  	<xsl:call-template name="bodyinnerend" />
	
	<!-- Occurences -->
	<xsl:for-each select="xsd:annotation/xsd:appinfo/elemAppInfo">
	 	<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalminoccurs" />
		<xsl:call-template name="logicalmaxoccurs" />
	  	<xsl:call-template name="bodyinnerend" />
	</xsl:for-each>	
	
	<!-- Documentation -->
<!--     <xsl:for-each select="xsd:element"> -->
	 	<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicaldocumentation" />
	  	<xsl:call-template name="bodyinnerend" />
<!--     </xsl:for-each> -->
  	
  	<xsl:call-template name="blockend" />

  </xsl:template>

	<!--
  ####   ######   ####   #    #  ######  #    #   ####   ######     
 #       #       #    #  #    #  #       ##   #  #    #  #          
  ####   #####   #    #  #    #  #####   # #  #  #       #####      
      #  #       #  # #  #    #  #       #  # #  #       #          
 #    #  #       #   #   #    #  #       #   ##  #    #  #          
  ####   ######   ### #   ####   ######  #    #   ####   ######     
-->
  <xsl:template match="xsd:sequence[xsd:annotation/xsd:appinfo/MRLocalGroup/@composition='orderedSet']">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquewildcard" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'OS'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$orderedSetString" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
  		<xsl:with-param name="icon" select="$icontype" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="displaytext" select="$orderedSetString" />
  	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicallocalcontentmodel" />
	<xsl:call-template name="logicallocalgroupcontent" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Occurences -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalminoccurs" />
	<xsl:call-template name="logicalmaxoccurs" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:apply-templates>
  </xsl:template>

  <xsl:template match="xsd:sequence[xsd:annotation/xsd:appinfo/MRLocalGroup/@composition='unorderedSet']">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquewildcard" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'US'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$unorderedSetString" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
  		<xsl:with-param name="icon" select="$icontype" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="displaytext" select="$unorderedSetString" />
  	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicallocalcontentmodel" />
	<xsl:call-template name="logicallocalgroupcontent" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Occurences -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalminoccurs" />
	<xsl:call-template name="logicalmaxoccurs" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:apply-templates>
  </xsl:template>
  
<!--
  ####   #    #   ####      #     ####   ######      
 #    #  #    #  #    #     #    #    #  #           
 #       ######  #    #     #    #       #####       
 #       #    #  #    #     #    #       #           
 #    #  #    #  #    #     #    #    #  #           
  ####   #    #   ####      #     ####   ######      
-->
  <xsl:template match="xsd:choice">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquewildcard" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'MGC'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$choiceString" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
  		<xsl:with-param name="icon" select="$icontype" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="displaytext" select="$choiceString" />
  	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicallocalcontentmodel" />
	<xsl:call-template name="logicallocalgroupcontent" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Occurences -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalminoccurs" />
	<xsl:call-template name="logicalmaxoccurs" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
   ##    #       #      
  #  #   #       #      
 #    #  #       #      
 ######  #       #      
 #    #  #       #      
 #    #  ######  ###### 
-->
  <xsl:template match="xsd:all">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquewildcard" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'MGA'" />

  	<xsl:variable name="displaypath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$displaysuffix" />
  			<xsl:with-param name="id" select="$allString" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="anchorpath">
  		<xsl:call-template name="appendtosuffix">
  			<xsl:with-param name="suffix" select="$anchorsuffix" />
  			<xsl:with-param name="id" select="$uniqueid" />
  		</xsl:call-template>
  	</xsl:variable>

  	<xsl:variable name="fullpath" select="concat($prefix,$anchorpath,$pathsep)" />

  	<xsl:call-template name="blockstart" />

  	<xsl:call-template name="writeAnchor">
  		<xsl:with-param name="icon" select="$icontype" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="displaytext" select="$allString" />
  	</xsl:call-template>

  	<xsl:call-template name="blockmiddle" />

 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicallocalcontentmodel" />
	<xsl:call-template name="logicallocalgroupcontent" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Occurences -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicalminoccurs" />
	<xsl:call-template name="logicalmaxoccurs" />
  	<xsl:call-template name="bodyinnerend" />

	<!-- Documentation -->
 	<xsl:call-template name="bodyinnerbegin" />
	<xsl:call-template name="logicaldocumentation" />
  	<xsl:call-template name="bodyinnerend" />
  	
  	<xsl:call-template name="blockend" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaypath" />
  		<xsl:with-param name="anchorsuffix" select="$anchorpath" />
  	</xsl:apply-templates>
  </xsl:template>
  <!--
 #####   ######   ####    #####  #####      #     ####    #####     #     ####  
 #    #  #       #          #    #    #     #    #    #     #       #    #    # 
 #    #  #####    ####      #    #    #     #    #          #       #    #    # 
 #####   #            #     #    #####      #    #          #       #    #    # 
 #   #   #       #    #     #    #   #      #    #    #     #       #    #    # 
 #    #  ######   ####      #    #    #     #     ####      #       #     ####  
-->
  <xsl:template match="xsd:restriction">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaysuffix" />
  		<xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
 ######  #    #   #####  ######  #    #   ####      #     ####   #    #
 #        #  #      #    #       ##   #  #          #    #    #  ##   #
 #####     ##       #    #####   # #  #   ####      #    #    #  # #  #
 #         ##       #    #       #  # #       #     #    #    #  #  # #
 #        #  #      #    #       #   ##  #    #     #    #    #  #   ##
 ######  #    #     #    ######  #    #   ####      #     ####   #    #
-->
  <xsl:template match="xsd:extension">
  	<xsl:param name="prefix" />
  	<xsl:param name="displaysuffix" />
  	<xsl:param name="anchorsuffix" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="displaysuffix" select="$displaysuffix" />
  		<xsl:with-param name="anchorsuffix" select="$anchorsuffix" />
  	</xsl:apply-templates>
  </xsl:template>

  <xsl:template name="processSimpleTypeFacets">
  
    <xsl:variable name="baseValue">
      <xsl:value-of select="xsd:restriction/@base"/>
    38866
    Done</xsl:variable>
  
    <xsl:variable name="localBaseValue">
      <xsl:choose>
        <xsl:when test="contains($baseValue,':')">
          <xsl:value-of select="substring-after($baseValue,':')" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$baseValue" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

  	<xsl:choose>
  		<xsl:when test="$localBaseValue='anyURI'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='base64Binary'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='ENTITY'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='hexBinary'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='ID'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='IDREF'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='language'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='Name'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='NCName'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='NMTOKEN'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='normalizedString'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='NOTATION'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='QName'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='string'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='token'">
  			<xsl:call-template name="processStringFacets" />
  		</xsl:when>
  		
  		<xsl:when test="$localBaseValue='ENTITIES'">
  			<xsl:call-template name="processStringFacetsNoPattern" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='IDREFS'">
  			<xsl:call-template name="processStringFacetsNoPattern" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='NMTOKENS'">
  			<xsl:call-template name="processStringFacetsNoPattern" />
  		</xsl:when>

  		<xsl:when test="$localBaseValue='boolean'">
  			<xsl:call-template name="processBooleanFacets" />
  		</xsl:when>

  		<xsl:when test="$localBaseValue='byte'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='decimal'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='int'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='integer'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='long'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='negativeInteger'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='nonNegativeInteger'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='nonPositiveInteger'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='positiveInteger'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='short'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='unsignedByte'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='unsignedInt'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='unsignedLong'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='unsignedShort'">
  			<xsl:call-template name="processIntFacets" />
  		</xsl:when>
  		
  		<xsl:when test="$localBaseValue='date'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='dateTime'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='double'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='duration'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='float'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='gDay'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='gMonth'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='gMonthDay'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='gYear'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='gYearMonth'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:when test="$localBaseValue='time'">
  			<xsl:call-template name="processIntFacetsNoDigits" />
  		</xsl:when>
  		<xsl:otherwise>
  		    <!-- Cannot determine the type -->
  		    <!-- Could just output whatever we find - but not yet -->
  		</xsl:otherwise>
  	</xsl:choose>
  </xsl:template>
  
	<xsl:template name="processIntFacets">
		<!-- Inclusive Constraints -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalinclusivemin" />
		<xsl:call-template name="logicalinclusivemax" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Exclusive Constraints -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalexclusivemin" />
		<xsl:call-template name="logicalexclusivemax" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Digits -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalfractiondigits" />
		<xsl:call-template name="logicaltotaldigits" />
		<xsl:call-template name="logicalwhitespace" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Enumerations -->
		<xsl:if test="xsd:restriction/xsd:enumeration">
			<xsl:call-template name="bodyinnerbegin" />
			<xsl:for-each select="xsd:restriction/xsd:enumeration">
				<xsl:call-template name="logicalenumeration" />
			</xsl:for-each>
			<xsl:call-template name="bodyinnerend" />
		</xsl:if>
		<!-- Patterns -->
		<xsl:if test="xsd:restriction/xsd:pattern">
			<xsl:call-template name="bodyinnerbegin" />
			<xsl:for-each select="xsd:restriction/xsd:pattern">
				<xsl:call-template name="logicalpattern" />
			</xsl:for-each>
			<xsl:call-template name="bodyinnerend" />
		</xsl:if>
	</xsl:template>

  <xsl:template name="processIntFacetsNoDigits">
		<!-- Inclusive Constraints -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalinclusivemin" />
		<xsl:call-template name="logicalinclusivemax" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Exclusive Constraints -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalexclusivemin" />
		<xsl:call-template name="logicalexclusivemax" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Enumerations -->
		<xsl:if test="xsd:restriction/xsd:enumeration">
			<xsl:call-template name="bodyinnerbegin" />
			<xsl:for-each select="xsd:restriction/xsd:enumeration">
				<xsl:call-template name="logicalenumeration" />
			</xsl:for-each>
			<xsl:call-template name="bodyinnerend" />
		</xsl:if>
		<!-- Patterns -->
		<xsl:if test="xsd:restriction/xsd:pattern">
			<xsl:call-template name="bodyinnerbegin" />
			<xsl:for-each select="xsd:restriction/xsd:pattern">
				<xsl:call-template name="logicalpattern" />
			</xsl:for-each>
			<xsl:call-template name="bodyinnerend" />
		</xsl:if>
  </xsl:template>

	<xsl:template name="processStringFacets">
		<!-- Length Constraints -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicallength" />
		<xsl:call-template name="logicalminlength" />
		<xsl:call-template name="logicalmaxlength" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Whitespace -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalwhitespace" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Enumerations -->
		<xsl:if test="xsd:restriction/xsd:enumeration">
			<xsl:call-template name="bodyinnerbegin" />
			<xsl:for-each select="xsd:restriction/xsd:enumeration">
				<xsl:call-template name="logicalenumeration" />
			</xsl:for-each>
			<xsl:call-template name="bodyinnerend" />
		</xsl:if>
		<!-- Patterns -->
		<xsl:if test="xsd:restriction/xsd:pattern">
			<xsl:call-template name="bodyinnerbegin" />
			<xsl:for-each select="xsd:restriction/xsd:pattern">
				<xsl:call-template name="logicalpattern" />
			</xsl:for-each>
			<xsl:call-template name="bodyinnerend" />
		</xsl:if>
	</xsl:template>

	<xsl:template name="processStringFacetsNoPattern">
		<!-- Length Constraints -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicallength" />
		<xsl:call-template name="logicalminlength" />
		<xsl:call-template name="logicalmaxlength" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Whitespace -->
		<xsl:call-template name="bodyinnerbegin" />
		<xsl:call-template name="logicalwhitespace" />
		<xsl:call-template name="bodyinnerend" />
		<!-- Enumerations -->
		<xsl:if test="xsd:restriction/xsd:enumeration">
			<xsl:call-template name="bodyinnerbegin" />
			<xsl:for-each select="xsd:restriction/xsd:enumeration">
				<xsl:call-template name="logicalenumeration" />
			</xsl:for-each>
			<xsl:call-template name="bodyinnerend" />
		</xsl:if>
	</xsl:template>

  <xsl:template name="processBooleanFacets">
  	<xsl:call-template name="logicalwhitespace" />
  	<xsl:for-each select="xsd:restriction/xsd:pattern">
  		<xsl:call-template name="logicalpattern" />
  	</xsl:for-each>
  </xsl:template>

  <!--
 #####    ####    ####   #    #  #    #  ######  #    #   #####    ##     ##### 
 #    #  #    #  #    #  #    #  ##  ##  #       ##   #     #     #  #      #   
 #    #  #    #  #       #    #  # ## #  #####   # #  #     #    #    #     #   
 #    #  #    #  #       #    #  #    #  #       #  # #     #    ######     #   
 #    #  #    #  #    #  #    #  #    #  #       #   ##     #    #    #     #   
 #####    ####    ####    ####   #    #  ######  #    #     #    #    #     #   
-->
  <xsl:template name="logicalabstract">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$abstractString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@abstract" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$falseString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalbasetype">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$basetypeString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="xsd:simpleContent/xsd:extension/@base">
  					<xsl:call-template name="lookupglobaltype">
  						<xsl:with-param name="type" select="xsd:simpleContent/xsd:extension/@base" />
  					</xsl:call-template>
  				</xsl:when>
  				<xsl:when test="xsd:complexContent/xsd:extension/@base">
  					<xsl:call-template name="lookupglobaltype">
  						<xsl:with-param name="type" select="xsd:complexContent/xsd:extension/@base" />
  					</xsl:call-template>
  				</xsl:when>
  				<xsl:when test="xsd:complexContent/xsd:restriction/@base">
  					<xsl:call-template name="lookupglobaltype">
  						<xsl:with-param name="type" select="xsd:complexContent/xsd:restriction/@base" />
  					</xsl:call-template>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:value-of select="$noneString" />
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalblock">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$blockString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@block" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalcomplexcontent">
  	<xsl:choose>
  		<xsl:when test="xsd:annotation/xsd:appinfo/MRComplexType/@composition">
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$localgroupString" />
  				<xsl:with-param name="value">
  					<xsl:value-of select="xsd:annotation/xsd:appinfo/MRComplexType/@composition" />
  				</xsl:with-param>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="xsd:all">
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$localgroupString" />
  				<xsl:with-param name="value">
  					<xsl:value-of select="'all'" />
  				</xsl:with-param>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="xsd:choice">
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$localgroupString" />
  				<xsl:with-param name="value">
  					<xsl:value-of select="'choice'" />
  				</xsl:with-param>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="xsd:sequence">
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$localgroupString" />
  				<xsl:with-param name="value">
  					<xsl:value-of select="'sequence'" />
  				</xsl:with-param>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="xsd:complexContent/*/xsd:sequence">
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$localgroupString" />
  				<xsl:with-param name="value">
  					<xsl:value-of select="'sequence'" />
  				</xsl:with-param>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="xsd:complexContent/*/xsd:choice">
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$localgroupString" />
  				<xsl:with-param name="value">
  					<xsl:value-of select="'choice'" />
  				</xsl:with-param>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="xsd:group[@ref]">
  			<xsl:call-template name="logicalgroupreference"/>
  		</xsl:when>
  		<xsl:otherwise>
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$localgroupString" />
  				<xsl:with-param name="value">
  					<!-- <xsl:value-of select="'&nbsp;'" /> -->
  					<xsl:value-of select="'empty'" />
  				</xsl:with-param>
  			</xsl:call-template>
  		</xsl:otherwise>
  	</xsl:choose>
  </xsl:template>

  <xsl:template name="logicalcomplextypecontent">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$typecontentString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:annotation/xsd:appinfo/*[local-name()='MRComplexType']/@content" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="'closed'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicaldefaultblock">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$defaultBlockString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@blockDefault" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicaldefaultfinal">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$defaultFinalString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@finalDefault" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicaldefaultorfixed">
  	<xsl:choose>
  		<xsl:when test="@fixed">
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$fixedString" />
  				<xsl:with-param name="value">
  					<xsl:value-of select="@fixed" />
  				</xsl:with-param>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:otherwise>
  			<xsl:call-template name="displayValue">
  				<xsl:with-param name="key" select="$defaultString" />
  				<xsl:with-param name="value">
  					<xsl:value-of select="@default" />
  				</xsl:with-param>
  				<xsl:with-param name="default" select="$noneString" />
  			</xsl:call-template>
  		</xsl:otherwise>
  	</xsl:choose>
  </xsl:template>

  <xsl:template name="logicalderivedby">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$derivedbyString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="xsd:complexContent/xsd:extension|xsd:simpleContent/xsd:extension">
  					<xsl:value-of select="$extensionString" />
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:value-of select="$restrictionString" />
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicaldocumentation">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$documentationString" />
  		<xsl:with-param name="value">
  			<xsl:call-template name="decode">
  				<xsl:with-param name="text" select="xsd:annotation/xsd:documentation" />
  			</xsl:call-template>
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicaldocumentationnotitle">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="'&nbsp;'" />
  		<xsl:with-param name="value">
  			<xsl:call-template name="decode">
  				<xsl:with-param name="text" select="xsd:annotation/xsd:documentation" />
  			</xsl:call-template>
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalembeddedname">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$nameString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:element/@type" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$unnamedString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalenumeration">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$enumerationString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalexclusivemax">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$exclusivemaxString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:maxExclusive/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalexclusivemin">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$exclusiveminString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:minExclusive/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalfinal">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$finalString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@final" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalfractiondigits">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$fractiondigitsString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:fractionDigits/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalfullname">
  	<xsl:param name="fullname"/>
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$fullnameString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="$fullname" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$unnamedString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalglobalcontentmodel">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$contentmodelString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
	  			<xsl:when test="xsd:annotation/xsd:appinfo/MRGlobalGroup/@composition">
		  			<xsl:value-of select="xsd:annotation/xsd:appinfo/MRGlobalGroup/@composition" />
	  			</xsl:when>
	  			<xsl:when test="xsd:choice">
	  				<xsl:value-of select="'choice'" />
	  			</xsl:when>
	  			<xsl:when test="xsd:sequence">
	  				<xsl:value-of select="'sequence'" />
	  			</xsl:when>
	  			<xsl:when test="xsd:all">
	  				<xsl:value-of select="'all'" />
	  			</xsl:when>
	  			<xsl:otherwise>
	  				<xsl:value-of select="'ERROR - UNEXPECTED CONTENT MODEL'" />
	  			</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  		<xsl:with-param name="default" select="'sequence'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalglobalgroupcontent">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$groupcontentString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:annotation/xsd:appinfo/MRGlobalGroup/@content" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="'closed'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalgroupreference">
	<xsl:call-template name="displayValue">
		<xsl:with-param name="key" select="$groupreferenceString" />
		<xsl:with-param name="value">
			<xsl:call-template name="lookupglobalgroup">
				<xsl:with-param name="group" select="xsd:group/@ref" />
			</xsl:call-template>
		</xsl:with-param>
	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalinclusivemax">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$inclusivemaxString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:maxInclusive/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalinclusivemin">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$inclusiveminString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:minInclusive/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicallength">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$lengthString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:length/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicallocalcomplextype">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$typeString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="$localComplexTypeString" />
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicallocalcomplextypecomplexcontent">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$typeString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="xsd:complexType/xsd:complexContent/xsd:extension/@base">
		  			<xsl:value-of select="concat(xsd:complexType/xsd:complexContent/xsd:extension/@base,' +')" />
  				</xsl:when>
  				<xsl:when test="xsd:complexType/xsd:complexContent/xsd:restriction/@base">
		  			<xsl:value-of select="concat(xsd:complexType/xsd:complexContent/xsd:restriction/@base,' -')" />
  				</xsl:when>
  			</xsl:choose>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicallocalcomplextypesimplecontent">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$typeString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="concat(xsd:complexType/xsd:simpleContent/xsd:extension/@base,' +')" />
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicallocalcontentmodel">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$contentmodelString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
	  			<xsl:when test="xsd:annotation/xsd:appinfo/MRLocalGroup/@composition">
		  			<xsl:value-of select="xsd:annotation/xsd:appinfo/MRLocalGroup/@composition" />
	  			</xsl:when>
	  			<xsl:otherwise>
	  				<xsl:value-of select="local-name()" />
	  			</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  		<xsl:with-param name="default" select="'sequence'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicallocalgroupcontent">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$groupcontentString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:annotation/xsd:appinfo/MRLocalGroup/@content" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="'closed'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicallocalsimpletype">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$typeString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
				<xsl:when test="xsd:simpleType/xsd:restriction/@base">
  					<xsl:value-of select="concat(xsd:simpleType/xsd:restriction/@base,' -')" />
				</xsl:when>
				<xsl:when test="xsd:simpleType/xsd:list/@itemType">
  					<xsl:value-of select="concat(xsd:simpleType/xsd:list/@itemType,' -')" />
				</xsl:when>
				<xsl:when test="xsd:simpleType/xsd:union">
  					<xsl:value-of select="concat('union',' -')" />
				</xsl:when>
  			</xsl:choose>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalmaxlength">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$maxString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:maxLength/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalmaxoccurs">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$maxoccursString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@maxOccurs" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="'1'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalmessagealias">
    <xsl:call-template name="displayValue">
        <xsl:with-param name="key" select="$messagealiasString" />
        <xsl:with-param name="value">
            <xsl:value-of select=".//@messageAlias" />
        </xsl:with-param>
        <xsl:with-param name="default" select="$noneString" />
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalminlength">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$minString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:minLength/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalminoccurs">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$minoccursString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@minOccurs" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="'1'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalmixed">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$mixedString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@mixed" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$falseString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalname">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$nameString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@name" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$unnamedString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalnamespace">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$namespaceString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@namespace" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalnamespacedefaultattribute">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$namespaceAttributeString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="@attributeFormDefault='qualified'">
  					<xsl:value-of select="$namespaceTarget"/>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:value-of select="$noTargetNamespaceString" disable-output-escaping="no"/>
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noTargetNamespaceString"/>
  		<xsl:with-param name="disableoutputescaping" select="'no'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalnamespacedefaultelement">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$namespaceElementString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="@elementFormDefault='qualified'">
  					<xsl:value-of select="$namespaceTarget"/>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:value-of select="$noTargetNamespaceString" disable-output-escaping="no"/>
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noTargetNamespaceString" />
  		<xsl:with-param name="disableoutputescaping" select="'no'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalnamespaceattribute">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$namespaceString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="count(ancestor::*)=1">
		  			<xsl:value-of select="$namespaceTarget"/>
  				</xsl:when>
  				<xsl:when test="@form='qualified'">
  					<xsl:value-of select="$namespaceTarget"/>
  				</xsl:when>
  				<xsl:when test="@form='unqualified'">
  					<xsl:value-of select="$noTargetNamespaceString" disable-output-escaping="no"/>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:choose>
  						<xsl:when test="/xsd:schema/@attributeFormDefault='qualified'">
		  					<xsl:value-of select="$namespaceTarget"/>
  						</xsl:when>
  						<xsl:otherwise>
		  					<xsl:value-of select="$noTargetNamespaceString" disable-output-escaping="no"/>
 						</xsl:otherwise>
  					</xsl:choose>
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noTargetNamespaceString" />
  		<xsl:with-param name="disableoutputescaping" select="'no'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalnamespaceelement">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$namespaceString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="count(ancestor::*)=1">
		  			<xsl:value-of select="$namespaceTarget"/>
  				</xsl:when>
  				<xsl:when test="@form='qualified'">
  					<xsl:value-of select="$namespaceTarget"/>
  				</xsl:when>
  				<xsl:when test="@form='unqualified'">
  					<xsl:value-of select="$noTargetNamespaceString"/>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:choose>
  						<xsl:when test="/xsd:schema/@elementFormDefault='qualified'">
		  					<xsl:value-of select="$namespaceTarget"/>
  						</xsl:when>
  						<xsl:otherwise>
		  					<xsl:value-of select="$noTargetNamespaceString"/>
 						</xsl:otherwise>
  					</xsl:choose>
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noTargetNamespaceString" />
  		<xsl:with-param name="disableoutputescaping" select="'no'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalnamespaceprefix">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$namespacePrefixString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="$namespacePrefix" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalnamespacetarget">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$namespaceTargetString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="$namespaceTarget" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalnillable">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$nillableString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@nillable" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$falseString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalpattern">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$patternString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalprocesscontents">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$processcontentsString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@processContents" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="'strict'" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalreferencename">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$referencenameString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@ref" />
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalreferencenameattribute">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$referencenameString" />
  		<xsl:with-param name="value">
  			<xsl:call-template name="lookupglobalattribute">
  				<xsl:with-param name="attribute" select="@ref" />
  			</xsl:call-template>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalreferencenameattributegroup">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$referencenameString" />
  		<xsl:with-param name="value">
  			<xsl:call-template name="lookupglobalattributegroup">
  				<xsl:with-param name="group" select="@ref" />
  			</xsl:call-template>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalreferencenameelement">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$referencenameString" />
  		<xsl:with-param name="value">
  			<xsl:call-template name="lookupglobalelement">
  				<xsl:with-param name="element" select="@ref" />
  			</xsl:call-template>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalreferencenamegroup">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$referencenameString" />
  		<xsl:with-param name="value">
  			<xsl:call-template name="lookupglobalgroup">
  				<xsl:with-param name="group" select="@ref" />
  			</xsl:call-template>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalsimpletypebasetype">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$basetypeString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="xsd:extension/@base">
  					<xsl:call-template name="lookupglobaltype">
  						<xsl:with-param name="type" select="xsd:extension/@base" />
  					</xsl:call-template>
  				</xsl:when>
  				<xsl:when test="xsd:restriction/@base">
  					<xsl:call-template name="lookupglobaltype">
  						<xsl:with-param name="type" select="xsd:restriction/@base" />
  					</xsl:call-template>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:value-of select="$noneString" />
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalsimpletypeitemtype">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$itemtypeString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="xsd:list/@itemType">
  					<xsl:call-template name="lookupglobaltype">
  						<xsl:with-param name="type" select="xsd:list/@itemType" />
  					</xsl:call-template>
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:value-of select="$noneString" />
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalsimpletypevariety">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$varietyString" />
  		<xsl:with-param name="value">
  			<xsl:choose>
  				<xsl:when test="xsd:restriction">
  					<xsl:value-of select="$atomicString" />
  				</xsl:when>
  				<xsl:when test="xsd:list">
  					<xsl:value-of select="$listString" />
  				</xsl:when>
  				<xsl:when test="xsd:union">
  					<xsl:value-of select="$unionString" />
  				</xsl:when>
  				<xsl:otherwise>
  					<xsl:value-of select="$noneString" />
  				</xsl:otherwise>
  			</xsl:choose>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalsubstitutiongroup">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$substitutiongroupString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@substitutionGroup" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicaltotaldigits">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$totaldigitsString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:totalDigits/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicaltype">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$typeString" />
  		<xsl:with-param name="value">
  			<xsl:call-template name="lookupglobaltype">
  				<xsl:with-param name="type" select="@type" />
  			</xsl:call-template>
  		</xsl:with-param>
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalmembertype">
      <xsl:param name="membertype"/>
    <xsl:call-template name="displayValue">
        <xsl:with-param name="key" select="$membertypeString" />
        <xsl:with-param name="value">
            <xsl:call-template name="lookupglobaltype">
                <xsl:with-param name="type" select="$membertype" />
            </xsl:call-template>
        </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalusage">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$usageString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="@use" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$optionalString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="logicalwhitespace">
  	<xsl:call-template name="displayValue">
  		<xsl:with-param name="key" select="$whitespaceString" />
  		<xsl:with-param name="value">
  			<xsl:value-of select="xsd:restriction/xsd:whiteSpace/@value" />
  		</xsl:with-param>
  		<xsl:with-param name="default" select="$noneString" />
  	</xsl:call-template>
  </xsl:template>

  <xsl:key name="globalentities" match="kvpair" use="@key"/>
  <xsl:variable name="globalentityfile" select="concat($outputPath,'/globals.lut.xml')"/>
  
  <xsl:template name="lookupglobal">
  	<xsl:param name="key" select="NoKeySupplied" />
  	<xsl:param name="text" select="NoTextSupplied" />
  	<xsl:for-each select="document($globalentityfile)/toc">
  		<xsl:choose>
  			<xsl:when test="key('lookupText',$key)/@value">
  				<xsl:text disable-output-escaping="yes">&lt;a href="</xsl:text>
  				<xsl:value-of select="$relativePath"/>
  				<xsl:value-of select="key('lookupText',$key)/@value" />
  				<xsl:text disable-output-escaping="yes">"&gt;</xsl:text>
  				<xsl:value-of select="$text" />
  				<xsl:text disable-output-escaping="yes">&lt;/a&gt;</xsl:text>
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="$text" />
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:for-each>
  </xsl:template>

	<!--
		Takes a namespace-qualified entity (eg ab:complexType1)
		and returns a namespace URI in the form http://a.b#complexType1
		Entities without a namespace (eg complexType1) are 
		returned as #complexType1
	-->
	<xsl:template name="expandNamespacePrefix">
		<xsl:param name="namespaceprefix" select="NoNamespacePrefixSupplied"/>
		<xsl:for-each select="/xsd:schema/namespace::*">
			<xsl:if test="local-name() = $namespaceprefix">
	  			<xsl:value-of select="self::node()"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	
  <xsl:template name="lookupglobalattribute">
  	<xsl:param name="attribute" select="NoAttributeSupplied" />
  	<xsl:call-template name="lookupglobalprologue">
  		<xsl:with-param name="entitytype" select="'elementsAndAttributes/attribute'" />
  		<xsl:with-param name="entityname" select="$attribute" /> 
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="lookupglobalattributegroup">
  	<xsl:param name="group" select="NoGroupSupplied" />
  	<xsl:call-template name="lookupglobalprologue">
  		<xsl:with-param name="entitytype" select="'groups/attributeGroup'" />
  		<xsl:with-param name="entityname" select="$group" /> 
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="lookupglobalelement">
  	<xsl:param name="element" select="NoElementSupplied" />
  	<xsl:call-template name="lookupglobalprologue">
  		<xsl:with-param name="entitytype" select="'elementsAndAttributes/element'" />
  		<xsl:with-param name="entityname" select="$element" /> 
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="lookupglobalgroup">
  	<xsl:param name="group" select="NoGroupSupplied" />
  	<xsl:call-template name="lookupglobalprologue">
  		<xsl:with-param name="entitytype" select="'groups/group'" />
  		<xsl:with-param name="entityname" select="$group" /> 
  	</xsl:call-template>
  </xsl:template>

  <xsl:template name="lookupglobaltype">
  	<xsl:param name="type" select="NoTypeSupplied" />
  	<xsl:call-template name="lookupglobalprologue">
  		<xsl:with-param name="entitytype" select="'types/type'" />
  		<xsl:with-param name="entityname" select="$type" /> 
  	</xsl:call-template>
  </xsl:template>
	
  <xsl:template name="lookupglobalprologue">
  	<xsl:param name="entitytype" select="NoEntityTypeSupplied"/>
  	<xsl:param name="entityname" select="NoEntityNameSupplied"/>
  	
  	<xsl:variable name="key">
  		<xsl:choose>
  			<xsl:when test="contains($entityname,':')">
  				<xsl:variable name="ns">
  					<xsl:call-template name="expandNamespacePrefix">
  						<xsl:with-param name="namespaceprefix" select="substring-before($entityname,':')"/>
  					</xsl:call-template>
  				</xsl:variable>
  				<xsl:variable name="nn" select="substring-after($entityname,':')"/>
  				<xsl:value-of select="concat($entitytype,'/',$ns,'#',$nn,$pathsep)"/>
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="concat($entitytype,'/#',$entityname,$pathsep)"/>
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>
  	
  	<xsl:call-template name="lookupglobal">
  		<xsl:with-param name="key" select="$key" />
  		<xsl:with-param name="text" select="$entityname" /> 
  	</xsl:call-template>
  </xsl:template>
<!--
 #####    ####    ####   #    #  #    #  ######  #    #   #####    ##     ##### 
 #    #  #    #  #    #  #    #  ##  ##  #       ##   #     #     #  #      #   
 #    #  #    #  #       #    #  # ## #  #####   # #  #     #    #    #     #   
 #    #  #    #  #       #    #  #    #  #       #  # #     #    ######     #   
 #    #  #    #  #    #  #    #  #    #  #       #   ##     #    #    #     #   
 #####    ####    ####    ####   #    #  ######  #    #     #    #    #     #   
 
All documentation is handled explicitly by other templates.
If called, this should be a noop.
-->
	<xsl:template match="xsd:documentation" />

<!--
   ##    #    #  #    #   ####    #####    ##     #####     #     ####   #    # 
  #  #   ##   #  ##   #  #    #     #     #  #      #       #    #    #  ##   # 
 #    #  # #  #  # #  #  #    #     #    #    #     #       #    #    #  # #  # 
 ######  #  # #  #  # #  #    #     #    ######     #       #    #    #  #  # # 
 #    #  #   ##  #   ##  #    #     #    #    #     #       #    #    #  #   ## 
 #    #  #    #  #    #   ####      #    #    #     #       #     ####   #    # 

All annotation is handled explicitly by other templates.
If called, this should be a noop.
-->
	<xsl:template match="xsd:annotation" />

<!--
 #####   ######   ####    ####   #####   ######     
 #    #  #       #    #  #    #  #    #  #          
 #    #  #####   #       #    #  #    #  #####      
 #    #  #       #       #    #  #    #  #          
 #    #  #       #    #  #    #  #    #  #          
 #####   ######   ####    ####   #####   ######     
-->
  <xsl:template name="decode">
  	<xsl:param name="text" />

  	<xsl:call-template name="decodespecial">
  		<xsl:with-param name="text">
  			<xsl:call-template name="decodespecial">
  				<xsl:with-param name="text">
  					<xsl:call-template name="decodespecial">
  						<xsl:with-param name="text">
  							<xsl:call-template name="decodespecial">
  								<xsl:with-param name="text" select="$text" />
  								<!--
	  							<xsl:call-template name="decodespecial">
  									<xsl:with-param name="text" select="$text" />
  									<xsl:with-param name="special" select="&lt;" />
  									<xsl:with-param name="replace">&amp;lt;</xsl:with-param>
  								</xsl:call-template>
  								-->
  								<xsl:with-param name="special" select="'\n'" />
  								<xsl:with-param name="replace">&lt;br/></xsl:with-param>
  							</xsl:call-template>
  						</xsl:with-param>
  						<xsl:with-param name="special" select="'\r'" />
  						<xsl:with-param name="replace" select="' '" />
  					</xsl:call-template>
  				</xsl:with-param>
  				<xsl:with-param name="special" select="'\t'" />
  				<xsl:with-param name="replace" select="' '" />
  			</xsl:call-template>
  		</xsl:with-param>
  		<xsl:with-param name="special" select="'\s'" />
  		<xsl:with-param name="replace" select="' '" />
  	</xsl:call-template>
  </xsl:template>

<!--
 #####   ######   ####    ####   #####   ######   ####   #####   ######   ####  
 #    #  #       #    #  #    #  #    #  #       #       #    #  #       #    # 
 #    #  #####   #       #    #  #    #  #####    ####   #    #  #####   #      
 #    #  #       #       #    #  #    #  #            #  #####   #       #      
 #    #  #       #    #  #    #  #    #  #       #    #  #       #       #    # 
 #####   ######   ####    ####   #####   ######   ####   #       ######   ####  
-->
  <xsl:template name="decodespecial">
  	<xsl:param name="text" select="'No text supplied.'" />
  	<xsl:param name="special" select="'x'" />
  	<xsl:param name="replace" select="'X'" />

  	<xsl:variable name="sb" select="substring-before($text,$special)" />
  	<xsl:variable name="sa" select="substring-after($text,$special)" />

  	<xsl:choose>
  		<xsl:when test="$sb and $sa">
  			<xsl:call-template name="decodespecial">
  				<xsl:with-param name="text" select="$sb" />
  				<xsl:with-param name="special" select="$special" />
				<xsl:with-param name="replace" select="$replace" />
  			</xsl:call-template>
  			<xsl:value-of disable-output-escaping="yes" select="$replace" />
  			<xsl:call-template name="decodespecial">
  				<xsl:with-param name="text" select="$sa" />
  				<xsl:with-param name="special" select="$special" />
				<xsl:with-param name="replace" select="$replace" />
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="$sb">
  			<xsl:call-template name="decodespecial">
  				<xsl:with-param name="text" select="$sb" />
  				<xsl:with-param name="special" select="$special" />
				<xsl:with-param name="replace" select="$replace" />
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="$sa">
  			<xsl:call-template name="decodespecial">
  				<xsl:with-param name="text" select="$sa" />
  				<xsl:with-param name="special" select="$special" />
				<xsl:with-param name="replace" select="$replace" />
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:otherwise>
  			<xsl:value-of disable-output-escaping="yes" select="$text" />
  		</xsl:otherwise>
  	</xsl:choose>
  </xsl:template>

	<xsl:template name="bodyinnerbegin">
		<xsl:text disable-output-escaping="yes">&lt;tr>&lt;td colspan="2" class="bodyinner">&lt;div class="bodyinner">&lt;table class="bodyinner">&lt;tbody>&eol;</xsl:text>
	</xsl:template>
	
	<xsl:template name="bodyinnerend">
		<xsl:text disable-output-escaping="yes">&lt;/tbody>&lt;/table>&lt;/div>&lt;/td>&lt;/tr>&eol;</xsl:text>
	</xsl:template>
	
	<!--
 #####   #        ####    ####   #    #   ####    #####    ##    #####    ##### 
 #    #  #       #    #  #    #  #   #   #          #     #  #   #    #     #   
 #####   #       #    #  #       ####     ####      #    #    #  #    #     #   
 #    #  #       #    #  #       #  #         #     #    ######  #####      #   
 #    #  #       #    #  #    #  #   #   #    #     #    #    #  #   #      #   
 #####   ######   ####    ####   #    #   ####      #    #    #  #    #     #   
-->
  <xsl:template name="blockstart">
  	<xsl:choose>
  		<xsl:when test="count(ancestor::*)=1 or local-name()='MRMessage'">
  			<xsl:text disable-output-escaping="yes">&lt;p>&lt;div class="outer">&lt;div class="globalhead">&eol;</xsl:text>
  		</xsl:when>
  		<xsl:otherwise>
  			<xsl:text disable-output-escaping="yes">&lt;p>&lt;div class="outer">&lt;div class="head">&eol;</xsl:text>
  		</xsl:otherwise>
  	</xsl:choose>
  </xsl:template>

  <!--
 #####   #        ####    ####   #    #  #    #     #    #####   #####   #      
 #    #  #       #    #  #    #  #   #   ##  ##     #    #    #  #    #  #      
 #####   #       #    #  #       ####    # ## #     #    #    #  #    #  #      
 #    #  #       #    #  #       #  #    #    #     #    #    #  #    #  #      
 #    #  #       #    #  #    #  #   #   #    #     #    #    #  #    #  #      
 #####   ######   ####    ####   #    #  #    #     #    #####   #####   ###### 
-->	
	<xsl:template name="blockmiddle">
  	  <xsl:choose>
  		<xsl:when test="count(ancestor::*)=1 or local-name()='MRMessage'">
	      <xsl:text disable-output-escaping="yes">&lt;/div>&eol;&lt;div class="globalbody">&lt;table class="body">&lt;tbody>&eol;</xsl:text>
  		</xsl:when>
  		<xsl:otherwise>
	      <xsl:text disable-output-escaping="yes">&lt;/div>&eol;&lt;div class="body">&lt;table class="body">&lt;tbody>&eol;</xsl:text>
  		</xsl:otherwise>
      </xsl:choose>
	</xsl:template>  
	
<!--
 #####   #        ####    ####   #    #  ######  #    #  #####    
 #    #  #       #    #  #    #  #   #   #       ##   #  #    #   
 #####   #       #    #  #       ####    #####   # #  #  #    #   
 #    #  #       #    #  #       #  #    #       #  # #  #    #   
 #    #  #       #    #  #    #  #   #   #       #   ##  #    #   
 #####   ######   ####    ####   #    #  ######  #    #  #####    
-->	
	<xsl:template name="blockend">
		<xsl:text disable-output-escaping="yes">&lt;/tbody>&lt;/table>&lt;/div>&lt;/div>&lt;/p>&eol;</xsl:text>
	</xsl:template>  
	
<!--
 #####      #     ####   #####   #         ##     #   #  #    #    ##    #      
 #    #     #    #       #    #  #        #  #     # #   #    #   #  #   #      
 #    #     #     ####   #    #  #       #    #     #    #    #  #    #  #      
 #    #     #         #  #####   #       ######     #    #    #  ######  #      
 #    #     #    #    #  #       #       #    #     #     #  #   #    #  #      
 #####      #     ####   #       ######  #    #     #      ##    #    #  ###### 
-->
  <xsl:template name="displayValue">
  	<xsl:param name="key">No Key</xsl:param>
  	<xsl:param name="value">No Value</xsl:param>
  	<xsl:param name="default">No Default</xsl:param>
  	<xsl:param name="disableoutputescaping">yes</xsl:param>

	<!--
  	<xsl:comment>
  		<xsl:text>displayValue</xsl:text>
  	</xsl:comment>
    -->
    
	<xsl:text disable-output-escaping="yes">&lt;tr>&eol;&lt;td class="left"></xsl:text>
	<xsl:value-of select="$key" />
	<xsl:text disable-output-escaping="yes">&lt;/td>&eol;&lt;td></xsl:text>
	<!--
	<xsl:value-of select="$disableoutputescaping"/>
	-->
	<xsl:choose>
		<xsl:when test="$value!=''">
			<xsl:choose>
				<xsl:when test="$disableoutputescaping='yes'">
					<xsl:value-of disable-output-escaping="yes" select="$value" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of disable-output-escaping="no" select="$value" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:otherwise>
			<xsl:choose>
				<xsl:when test="$disableoutputescaping='yes'">
					<xsl:value-of disable-output-escaping="yes" select="$default" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of disable-output-escaping="no" select="$default" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:text disable-output-escaping="yes">&lt;/td>&eol;&lt;/tr>&eol;</xsl:text>

  </xsl:template>

<!--
 #    #  #####      #     #####  ######    ##    #    #   ####   #    #   ####  
 #    #  #    #     #       #    #        #  #   ##   #  #    #  #    #  #    # 
 #    #  #    #     #       #    #####   #    #  # #  #  #       ######  #    # 
 # ## #  #####      #       #    #       ######  #  # #  #       #    #  #    # 
 ##  ##  #   #      #       #    #       #    #  #   ##  #    #  #    #  #    # 
 #    #  #    #     #       #    ######  #    #  #    #   ####   #    #   ####  
-->
  <xsl:template name="writeAnchor">
  	<xsl:param name="icon" />
  	<xsl:param name="target" />
  	<xsl:param name="displaytext" />

  	<xsl:variable name="iconname">
  		<xsl:choose>
  			<xsl:when test="$icon='AD'">
  				<xsl:text>XSDAttributeDeclaration</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='AGD'">
  				<xsl:text>XSDAttributeGroupDefinition</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='AGU'">
  				<xsl:text>XSDAttributeGroupUse</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='AU'">
  				<xsl:text>XSDAttributeUse</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='CTD'">
  				<xsl:text>XSDComplexTypeDefinition</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='ED'">
  				<xsl:text>XSDElementDeclaration</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='EST'">
  				<xsl:text>embedded_simpletypedefinition_obj</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='EU'">
  				<xsl:text>XSDElementUse</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='MDC'">
  				<xsl:text>msgdefcontainer_obj</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='MGA'">
  				<xsl:text>XSDModelGroupAll</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='MGC'">
  				<xsl:text>XSDModelGroupChoice</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='MGD'">
  				<xsl:text>XSDModelGroupDefinition</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='MGS'">
  				<xsl:text>XSDModelGroupSequence</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='MGU'">
  				<xsl:text>XSDModelGroupUse</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='MO'">
  				<xsl:text>message_obj</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='OS'">
  				<xsl:text>orderedset_cm</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='SBT'">
  				<xsl:text>simplebasetype_obj</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='STDA'">
  				<xsl:text>XSDSimpleTypeDefinitionAtomic</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='STDL'">
  				<xsl:text>XSDSimpleTypeDefinitionList</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='STDU'">
  				<xsl:text>XSDSimpleTypeDefinitionUnion</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='US'">
  				<xsl:text>unorderedset_cm</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='WA'">
  				<xsl:text>XSDWildcardAttribute</xsl:text>
  			</xsl:when>
  			<xsl:when test="$icon='WE'">
  				<xsl:text>XSDWildcardElement</xsl:text>
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:text>No Icon Defined</xsl:text>
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>

	<xsl:text disable-output-escaping="yes">&lt;a name="</xsl:text>
	<xsl:value-of select="$target" disable-output-escaping="yes"/>
	<xsl:text disable-output-escaping="yes">"&gt;&eol;</xsl:text>

	<xsl:text disable-output-escaping="yes">&lt;img alt="</xsl:text>
    <xsl:value-of select="$iconname" />
	<xsl:text disable-output-escaping="yes">" src="</xsl:text>
    <xsl:value-of select="$relativePath" />
	<xsl:text disable-output-escaping="yes">images/</xsl:text>
    <xsl:value-of select="$iconname" />
	<xsl:text disable-output-escaping="yes">.gif" /&gt;&eol;</xsl:text>
	
	<xsl:text disable-output-escaping="yes">&lt;/a&gt;&eol;</xsl:text>

	<xsl:value-of disable-output-escaping="yes" select="$displaytext" />
  		

  </xsl:template>

<!--
 #    #  #####      #     #####  ######   ####   ######   ####    #####     #  
 #    #  #    #     #       #    #       #       #       #    #     #       #  
 #    #  #    #     #       #    #####    ####   #####   #          #       #  
 # ## #  #####      #       #    #            #  #       #          #       #  
 ##  ##  #   #      #       #    #       #    #  #       #    #     #       #  
 #    #  #    #     #       #    ######   ####   ######   ####      #       #  
-->
  <xsl:template name="writeSectionAnchor">
  	<xsl:param name="target" />
  	<xsl:param name="displaytext" />

	<xsl:text disable-output-escaping="yes">&lt;a name="</xsl:text>
	<xsl:value-of select="$target" disable-output-escaping="yes"/>
	<xsl:text disable-output-escaping="yes">"&gt;&eol;</xsl:text>

	<xsl:text disable-output-escaping="yes">&lt;h4&gt;</xsl:text>
	<xsl:value-of select="$displaytext" />
	<xsl:text disable-output-escaping="yes">&lt;/h4&gt;&eol;</xsl:text>

	<xsl:text disable-output-escaping="yes">&lt;/a&gt;&eol;</xsl:text>

  </xsl:template>

<!--
 #####   #        ####    ####   #    #   ####    ####   #    #  #    #  ###### 
 #    #  #       #    #  #    #  #   #   #    #  #    #  ##  ##  ##  ##  #      
 #####   #       #    #  #       ####    #       #    #  # ## #  # ## #  #####  
 #    #  #       #    #  #       #  #    #       #    #  #    #  #    #  #      
 #    #  #       #    #  #    #  #   #   #    #  #    #  #    #  #    #  #      
 #####   ######   ####    ####   #    #   ####    ####   #    #  #    #  ###### 
-->
	<xsl:template name="blockcomment">
		<xsl:param name="text">No comment</xsl:param>
		<xsl:text disable-output-escaping="yes">&eol;&lt;!--&eol;====================================&eol;</xsl:text>
		<xsl:value-of select="$text"/>
		<xsl:text disable-output-escaping="yes">&eol;====================================&eol;-->&eol;</xsl:text>
	</xsl:template>  

<!--
 #    #  #    #     #     ####   #    #  ######  #    #    ##    #    #  ###### 
 #    #  ##   #     #    #    #  #    #  #       ##   #   #  #   ##  ##  #      
 #    #  # #  #     #    #    #  #    #  #####   # #  #  #    #  # ## #  #####  
 #    #  #  # #     #    #  # #  #    #  #       #  # #  ######  #    #  #      
 #    #  #   ##     #    #   #   #    #  #       #   ##  #    #  #    #  #      
  ####   #    #     #     ### #   ####   ######  #    #  #    #  #    #  ###### 
-->
    <xsl:template name="uniquename">
    	<xsl:variable name="nances" select="count(ancestor::*)" />
    	<xsl:if test="$nances=1">
    	<!--
    		<xsl:variable name="atname" select="@name" />
    		<xsl:variable name="myname" select="name()" />
    		<xsl:variable name="count" select="count(preceding-sibling::*[@name=$atname and name()=$myname])" />
    		<xsl:value-of select="concat($atname,'(',$count,')')" />
    		-->
    		<xsl:value-of select="@name" />
    	</xsl:if>
    	<xsl:if test="$nances!=1">
    		<xsl:value-of select="generate-id()" />
    	</xsl:if>
    </xsl:template>

    <xsl:template name="uniquemembertypeid">
        <xsl:param name="membertype"/>
        <xsl:value-of select="concat(generate-id(),'.',$membertype)" />
    </xsl:template>

<!--
 #    #  #    #     #     ####   #    #  ######  #####   ######  ######
 #    #  ##   #     #    #    #  #    #  #       #    #  #       #
 #    #  # #  #     #    #    #  #    #  #####   #    #  #####   #####
 #    #  #  # #     #    #  # #  #    #  #       #####   #       #
 #    #  #   ##     #    #   #   #    #  #       #   #   #       #
  ####   #    #     #     ### #   ####   ######  #    #  ######  #
-->
    <xsl:template name="uniqueref">
    	<xsl:variable name="nances" select="count(ancestor::*)" />
    	<xsl:if test="$nances=1">
    		<xsl:variable name="atname" select="@ref" />
    		<xsl:variable name="myname" select="name()" />
    		<xsl:variable name="count" select="count(preceding-sibling::*[@ref=$atname and name()=$myname])" />
    		<xsl:value-of select="concat($atname,'(',$count,')')" />
    	</xsl:if>
    	<xsl:if test="$nances!=1">
    		<xsl:value-of select="generate-id()" />
    	</xsl:if>
    </xsl:template>

<!--
 #    #  #    #     #     ####   #    #  ######  #    #     #    #       #####
 #    #  ##   #     #    #    #  #    #  #       #    #     #    #       #    #
 #    #  # #  #     #    #    #  #    #  #####   #    #     #    #       #    #
 #    #  #  # #     #    #  # #  #    #  #       # ## #     #    #       #    #
 #    #  #   ##     #    #   #   #    #  #       ##  ##     #    #       #    #
  ####   #    #     #     ### #   ####   ######  #    #     #    ######  #####
-->
    <xsl:template name="uniquewildcard">
    	<xsl:variable name="nances" select="count(ancestor::*)" />
    	<xsl:if test="$nances=1">
    		<xsl:variable name="myname" select="name()" />
    		<xsl:variable name="count" select="count(preceding-sibling::*[name()=$myname])" />
    		<xsl:value-of select="concat(local-name(),'(',$count,')')" />
    	</xsl:if>
    	<xsl:if test="$nances!=1">
    		<xsl:value-of select="generate-id()" />
    	</xsl:if>
    </xsl:template>

<!-- 
   ##    #####   #####   ######  #    #  #####    #####   ####    ####   #    #
  #  #   #    #  #    #  #       ##   #  #    #     #    #    #  #       #    #
 #    #  #    #  #    #  #####   # #  #  #    #     #    #    #   ####   #    #
 ######  #####   #####   #       #  # #  #    #     #    #    #       #  #    #
 #    #  #       #       #       #   ##  #    #     #    #    #  #    #  #    #
 #    #  #       #       ######  #    #  #####      #     ####    ####    ####
-->
    <xsl:template name="appendtosuffix">
		<xsl:param name="suffix"/>
		<xsl:param name="id"/>
		<xsl:choose>
			<xsl:when test="$suffix">
	  			<xsl:value-of select="concat($suffix,$pathsep,$id)" />
	  		</xsl:when>
	  		<xsl:otherwise>
	  			<xsl:value-of select="$id" />
	  		</xsl:otherwise>
  		</xsl:choose>
	</xsl:template>

  <!-- The NLS processing is common between the stylesheets -->
  <xsl:include href="docgen_nls.xsl"/>

</xsl:stylesheet>
